package com.scrdev.pg.kokotimeapp.series.addons;

import com.scrdev.pg.kokotimeapp.player.VideoSource;
import com.scrdev.pg.kokotimeapp.series.EpisodeItem;
import com.scrdev.pg.kokotimeapp.series.SeriesItem;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;


/**
 * 
 * 
 * @author Scrdev part of Kokotime copyright 2017
 *
 * All methods that return ArrayList<SeriesItem> should not return null, if nno array is
 * available simply return an empty ArrayList
 * 
 * All methods that return ArrayList of SeriesItem need to have the following values set to non null
 * values :
 * SeriesItem.seriesName
 * SeriesItem.seriesIconLink
 * the {@link #getUpcomingEpisodes()} needs to also set SeriesItem.upcommingDate and SeriesItem.upcommingEpisode
 *
 */

public interface SeriesAddonInterface {

	/**
	 * Called by the home page to generate the top featured pager
	 * @return ArrayList<SeriesItem> each SeriesItem must set the seriesName, seriesIconLink.
	 * @throws IOException you should handle other exceptions
	 */
    ArrayList<SeriesItem> getFeatured() throws IOException;

    
    /**
     * Called by homepage to generate the main page tabs 
     * 
     * @return String[] of the genres to display in the tabs, each genre in the array will
     * be passed one by one as a parameter to @see {@link #getPopularSeries(String)}
     */
    String[] getGenreArray();

    /**
     * Called by main page to generate the list of popular series of a specific genre.
     * 
     * @param GENRE_TO_SET = a genre from the array you returned in @see {@link #getGenreArray()}
     * @return ArrayList<SeriesItem> each SeriesItem must set the seriesName, seriesIconLink.
     * @throws IOException
     */
    ArrayList<SeriesItem> getPopularSeries(String GENRE_TO_SET) throws IOException;

    /**
     * Called by any part of the app that needs an mp4 source url
     * @param seriesItem that was filled in in the @see {@link #fillSeriesInfo(SeriesItem)}, the requested season and episode 
     * are stored in SeriesItem.lastSeenSeason and SeriesItem.lastSeenEpisode
     * @return ArrayList<VideoSource> of mp4 VideoSource objects, the url must contain protocols http:// or https://
     * @throws IOException
     */
    ArrayList<VideoSource> getVideoSources(SeriesItem seriesItem) throws IOException;

    /**
     * Called by the Series info page, to display information about the series
     * 
     * @param item the SeriesItem returned by you in one of the 
     * following methods {@link #getPopularSeries(String)}, {@link #getRecentlyAdded()}, {@link #getUpcomingEpisodes()}, {@link #getFeatured()}
     * @return SeriesItem passed along as parameter to which you will set non null values to SeriesItem.releaseDate , SeriesItem.description, 
     * SeriesItem.rating , SeriesItem.seasonCount, if these values were previously set then simply return the passed parameter.
     * @throws IOException
     */
    SeriesItem fillSeriesInfo(SeriesItem item) throws IOException;

    
    /**
     * Called by the Series info page for each season
     * 
     * @param seriesItem the currently displayed seriesItem, returned by {@link #fillSeriesInfo(SeriesItem)} 
     * @param season the season for which to return the episodes, same as the one in seriesItem.lastSeenSeason
     * @return ArrayList<EpisodeItem>m, the variables not set in the constructor can be left null
     * @throws IOException
     */
    ArrayList<EpisodeItem> getEpisodes(SeriesItem seriesItem, int season) throws IOException;

    
    /**
     * Called by recently added page
     * 
     * @return ArrayList<SeriesItem> of Series recently added to the service and available 
     * @throws IOException
     */
    ArrayList<SeriesItem> getRecentlyAdded() throws IOException;

    
    /**
     * Called by upcomming page
     * 
     * @return ArrayList<SeriesItem> of upcomming series, appart from the constructor variables of SeriesItem you should also set
     * non null values to SeriesItem.upcomingDate and SeriesItem.upcommingEpisode
     * @throws IOException
     */
    ArrayList<SeriesItem> getUpcomingEpisodes() throws IOException;
    
    

    /**
     * Called upon search in search page
     * 
     * @param query search text enterred by user
     * @return ArrayList<SeriesItem> containing the results obtained by your server for the search query
     * @throws IOException
     */
    ArrayList<SeriesItem> searchSeries(String query) throws IOException;
}
